@@ -1,9 +1,62 @@
+package Games::LMSolve::Input::Scalar::FH;
+
+sub TIEHANDLE
+{
+    my $class = shift;
+    my $self = {};
+    my $buffer = shift;
+    $self->{'lines'} = [ reverse(my @a = ($buffer =~ /([^\n]*(?:\n|$))/sg)) ];
+    bless $self, $class;
+    return $self;
+}
+
+sub READLINE
+{
+    my $self = shift;
+    return pop(@{$self->{'lines'}});
+}
+
+sub EOF
+{
+    my $self = shift;
+    return (scalar(@{$self->{'lines'}}) == 0);
+}
+
 package Games::LMSolve::Input;
 
 use strict;
 
 use English;
 
+=head1 NAME
+
+Games::LMSolve::Input - input class for LM-Solve
+
+=head1 SYNOPSIS
+
+    use Games::LMSolve::Input;
+
+    my $input_obj = Games::LMSolve::Input->new();
+
+    my $file_spec = "board.txt";
+
+    my $spec =
+    {
+        'dims' => { 'type' => "xy(integer)", 'required' => 1, },
+        'planks' => { 'type' => "array(start_end(xy(integer)))", 
+                      'required' => 1,
+                    },
+        'layout' => { 'type' => "layout", 'required' => 1,},        
+    };
+
+    my $input_fields = $input_obj->input_board($filename, $spec);
+
+=head1 DESCRIPTION
+
+This class implements the C<input_board> method, which enables to read boards
+in "key = value" format. Several types of values are supported.
+=cut
+
 sub new
 {
     my $class = shift;
@@ -24,45 +77,115 @@ sub initialize
     return 0;
 }
 
+=head1 $self->input_board($file_spec, $spec);
+
+This method accepts two arguments. C<$file_spec> which is the filename, 
+reference to a filehandle, or reference to the text containing the board 
+specification.
+
+$spec is a specification of the board given as a reference to a hash. 
+The keys are the keys inside the file. The values are references
+to hashes containing parameters. The 'required' parameter is given to
+specify that an exception should be thrown if this key was not specified. The
+other parameter (a mandatory one) is type which specified the type of the 
+value. Available types are:
+
+=over 8
+
+=item integer
+
+A simple integer (will be returned as a scalar)
+
+=item xy(integer)
+
+An (X,Y) pair. Will be returned as { 'x' => $x, 'y' => $y }.
+
+=item array(xy(integer))
+
+An array of [(X1,Y1),(X2,Y2),(X3,Y3)...] pairs. Will be returned as a
+reference to an array of (X,Y) pairs.
+
+=item array(start_end(xy(integer)))
+
+An array of [((SX1,SX2)->(EX1,EX2)), ((SX1,SX2)->(EX1,EX2))...] pairs of 
+(X,Y) pairs. Will be returned as a reference to an array of 
+
+    { 
+        'start' => { 'x' => $start_x, 'y' => $start_y },
+        'end' => { 'x' => $end_x, 'y' => $end_y },
+    }
+
+=item layout
+
+This is a generic layout that comes inside a here-document. It is returned
+as an array of lines that later have to be processed by another routine.
+
+=back
+
+=cut
+
 sub input_board
 {
     my $self = shift;
 
-    my $filename = shift;
+    my $file_spec = shift;
 
     my $spec = shift;
 
     my $ret = {};
 
+    my $file_ref;
+
     local (*I);
 
-    open(I, "<$filename") ||
-        die "Failed to read \"$filename\" : $OS_ERROR";
+    my $filename_str;
 
-    # Now we have the filehandle "I" opened.
+    if (ref($file_spec) eq "")
+    {
+        my $filename = $file_spec;
+        open(I, "<$filename") ||
+            die "Failed to read \"$filename\" : $OS_ERROR";
+
+        $file_ref = \*I;
+        $filename_str = ($filename eq "-") ? 
+            "standard input" : 
+            "\"$filename\"";
+    }
+    elsif (ref($file_spec) eq "GLOB")
+    {
+        $file_ref = $file_spec;
+        $filename_str = "FILEHANDLE";
+    }
+    elsif (ref($file_spec) eq "SCALAR")
+    {
+        tie(*I, "Games::LMSolve::Input::Scalar::FH", $$file_spec);
+        $file_ref = \*I;
+        $filename_str = "BUFFER";
+    }
+    else
+    {
+        die "Unknown file specification passed to input_board!";
+    }
+    
+    # Now we have the filehandle *$file_ref opened.
 
     my $line;
     my $line_num = 0;
 
     my $read_line = sub {
-        if (eof(I))
+        if (eof(*{$file_ref}))
         {
             return 0;
         }
-        $line = <I>;
+        $line = readline(*{$file_ref});
         $line_num++;
         chomp($line);
         return 1;
     };
 
-    my $filename_str = 
-        ($filename eq "-") ? 
-            "standard input" : 
-            "\"$filename\"";
-
     my $gen_exception = sub {
         my $text = shift;
-        close(I);
+        close(*{$file_ref});
         die "$text on $filename_str at line $line_num!\n";
     };
 
@@ -199,7 +322,7 @@ sub input_board
         }
     }
 
-    close(I);
+    close(*{$file_ref});
 
     foreach my $key (keys(%$spec))
     {
@@ -280,6 +403,12 @@ sub input_horiz_vert_walls_layout
     return (\@horiz_walls, \@vert_walls);
 }
 
+=head1 AUTHORS
+
+Written by Shlomi Fish E<lt>shlomif@vipe.technion.ac.ilE<gt>
+
+=cut
+
 1;
 
 
