@@ -77,14 +77,6 @@ sub display_state
     return sprintf("($x,$y)");
 }
 
-sub check_if_unsolveable
-{
-    my $self = shift;
-    my $coords = shift;
-
-    return 0;
-}
-
 sub check_if_final_state
 {
     my $self = shift;
